
/*This code illustrates the bandwidth limit feature. It configures an MDMA to read from DMC and measures
   the throughput for different values of BWLCNT register.
 */

#include <ccblkfn.h>
#include "adi_initialize.h"
#include "Bandwidth_Limit.h"
#include <cdefBF707.h>
#include <defBF707.h>
#include "time.h"
#include "stdio.h"
#include "math.h"

#pragma align 32
section("L1_data_a") unsigned char ucIntDestBuff[MAX_SIZE];  //Destination buffer in internal memory
section("sdram0_bank0") unsigned char ucExtSrcBuff[MAX_SIZE]; //Source buffer in external (DDR2) memory

//Global variables
volatile clock_t clock_start;
volatile clock_t clock_stop_mdma1;
volatile clock_t cycles_mdma1;
float throughput_mdma1;
int DMA_Done_MDMA1=0;

//DMA buffer size = 16 K bytes
unsigned int uiBytes=16384;

void MDMA1_Interrupt_Handler(uint32_t iid, void* handlerArg);

//Macro to configure MDMA
#define MDMA_CONFIG(S_MDMA_CHANNEL,S_CFG, S_ADDRSTART, S_MOD, S_COUNT,D_MDMA_CHANNEL,D_CFG, D_ADDRSTART, D_MOD, D_COUNT)\
		*pREG_DMA##S_MDMA_CHANNEL##_CFG = S_CFG;\
		*pREG_DMA##S_MDMA_CHANNEL##_ADDRSTART = S_ADDRSTART;\
		*pREG_DMA##S_MDMA_CHANNEL##_XCNT = S_COUNT;\
		*pREG_DMA##S_MDMA_CHANNEL##_XMOD = S_MOD;\
		*pREG_DMA##D_MDMA_CHANNEL##_CFG = D_CFG;\
		*pREG_DMA##D_MDMA_CHANNEL##_ADDRSTART = D_ADDRSTART;\
		*pREG_DMA##D_MDMA_CHANNEL##_XCNT = D_COUNT;\
		*pREG_DMA##D_MDMA_CHANNEL##_XMOD = D_MOD;

//Macro to wait for DMA done
#define WAIT_FOR_DMADONE(DMA_CHANNEL)\
		while(!(*pREG_DMA##DMA_CHANNEL##_STAT & ENUM_DMA_STAT_IRQDONE));

//Macro to disable the MDMA
#define DISABLE_DMAs(S_MDMA_CHANNEL, D_MDMA_CHANNEL)\

int main(void)
{

	//Enable MDMA1 stream as secure master
	*pREG_SPU0_SECUREP53=0x3;
	*pREG_SPU0_SECUREP54=0x3;

	//Program clock domain crossing registers in SYNC mode
	*pREG_SCB0_MST30_SYNC=0;
	*pREG_SCB0_MST31_SYNC=0;
	*pREG_SCB0_IB7_SYNC=0;

	//Change the DMC efficiency controller settings here
	*pREG_DMC0_CTL|=BITM_DMC_CTL_PPREF;
	*pREG_DMC0_EFFCTL|=(0xF<<BITP_DMC_EFFCTL_NUMREF)|(0xF<<BITP_DMC_EFFCTL_IDLECYC);

	//Initialize iterrupts and handlers
	*pREG_SEC0_GCTL=ENUM_SEC_GCTL_EN;
	*pREG_SEC0_CCTL0=ENUM_SEC_CCTL_EN;
	adi_int_InstallHandler(INTR_MDMA1_DST, MDMA1_Interrupt_Handler, NULL, true);

	//Configure MDMA 18-19 channels to read from DMC and
	MDMA_CONFIG(18, SOURCE_DMA_CFG,DDR2_BANK0_START_ADDRESS, SOURCE_MSIZE, uiBytes/SOURCE_MSIZE,19,DEST_DMA_CFG, ucIntDestBuff, DEST_MSIZE, uiBytes/DEST_MSIZE);

	//Configure bandwidth limit register here (BWLCNT = SCLK speed in MHz*MSIZE in bytes/Bandwidth required in MB/s)
	*pREG_DMA18_BWLCNT=8;

	//Enable DMAs now
	*pREG_DMA18_CFG |= 0x1;
	*pREG_DMA19_CFG |= 0x1;

	//Start the cycle count
	clock_start = clock();

	//Wait till DMA done
	while(DMA_Done_MDMA1==0);

	//Calculate the number of cycles to complete MDMA
	cycles_mdma1=clock_stop_mdma1-clock_start;

	//Disable the DMAs here
	DISABLE_DMAs(18,19);

	//Calculate and print the measured throughput
	throughput_mdma1=(float)(uiBytes*CCLK_SPEED/cycles_mdma1);
	printf("\n\n No of bytes=%d, No of cycles for MDMA1 =%d\n\n ",uiBytes,cycles_mdma1);
	printf("Throughput MDMA1=%f MB/s\n\n",throughput_mdma1);

	return 0;
}


void MDMA1_Interrupt_Handler(uint32_t iid, void* handlerArg)
{
	//Stop the cycle count
	clock_stop_mdma1 = clock();

	*pREG_DMA19_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit

	DMA_Done_MDMA1 = 1;
}


